\name{ipred-internal}
\alias{getsurv}
\title{Internal ipred functions}
\description{
 Internal ipred functions.
}
\usage{
getsurv(obj, times)
}
\details{
  This functions are not to be called by the user.
}
\keyword{internal}
